/*****************************************************************************
 * gyach_int_xtext.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2000-2002 Chris Pinkham
 * Released under the terms of the GPL.
 * *NO WARRANTY*
 *
 * cpinkham@infi.net, cpinkham@bc2va.org
 * http://www4.infi.net/~cpinkham/gyach/
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#ifdef GYACH_INT_TEXT

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#ifdef USE_GDK_PIXBUF
#include <gdk-pixbuf/gdk-pixbuf.h>
#endif

#ifdef USE_PTHREAD_CREATE
#include <pthread.h>
#endif

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "gyach.h"
#include "gyach_int.h"

GtkText *chat_text;
GdkFont *chat_font = NULL;
GdkColor chat_color;
int max_lines = 0;

/* support functiond decls */
gboolean on_chat_text_clicked (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
gint chat_motion_notify(GtkWidget * widget, GdkEventMotion * event);

/* standard gyach interface function decls */
int ct_build() {
	GtkWidget *hbox;
	GtkWidget *vscrollbar;

	hbox = lookup_widget( chat_window, "hbox2" );

	chat_text = GTK_TEXT(gtk_text_new( NULL, NULL ));
	gtk_widget_ref( GTK_WIDGET(chat_text) );
	gtk_object_set_data_full( GTK_OBJECT(hbox), "chat_text",
		chat_text, (GtkDestroyNotify)gtk_widget_unref );
	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(chat_text), TRUE, TRUE, 0);
	gtk_widget_set_usize( GTK_WIDGET(chat_text), 480, -2);
	gtk_widget_show( GTK_WIDGET(chat_text) );

	gtk_signal_connect_after (GTK_OBJECT (chat_text), "button_press_event",
		GTK_SIGNAL_FUNC (on_chat_text_clicked), NULL);

	gtk_signal_connect_after (GTK_OBJECT (chat_text), "motion_notify_event",
		GTK_SIGNAL_FUNC (chat_motion_notify), NULL);

	GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(chat_text), GTK_CAN_FOCUS);

	gtk_widget_set_events( GTK_WIDGET(chat_text),
		GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK );

	vscrollbar = gtk_vscrollbar_new( chat_text->vadj );
	gtk_box_pack_start (GTK_BOX (hbox), vscrollbar, FALSE, FALSE, 1);
	gtk_widget_show( vscrollbar );

	chat_font = gdk_font_load( "fixed" );

	chat_color.red = 0;
	chat_color.green = 0;
	chat_color.blue = 0;

	gdk_colormap_alloc_color( gtk_widget_get_colormap( chat_window ),
		&chat_color, FALSE, TRUE );

	gtk_text_set_word_wrap( chat_text, 1 );
	gtk_text_set_line_wrap( chat_text, 1 );

	return( 0 );
}

GtkWidget *ct_widget() {
	return( GTK_WIDGET(chat_text));
}

int ct_scroll_at_bottom() {
	GtkAdjustment *adj;

	adj = chat_text->vadj;
	if ( adj->value == ( adj->upper - adj->lower - adj->page_size ))
		return( 1 );
	else
		return( 0 );
}

int ct_scroll_to_bottom() {
	GtkAdjustment *adj;

	adj = chat_text->vadj;
	gtk_adjustment_set_value( adj,
		adj->upper - adj->lower - adj->page_size );

	return( 0 );
}

int remove_extra_lines() {
	int text_length;

	text_length = gtk_text_get_length( chat_text );

	if ( max_lines ) {
		ct_freeze();
		if ( text_length > ( max_lines * 1024 )) {
			gtk_text_set_point( chat_text, 0 );
			gtk_text_forward_delete( chat_text, 512 );

			text_length = gtk_text_get_length( chat_text );
			gtk_text_set_point( chat_text, text_length );
		}
		ct_thaw();
	}

	return( 0 );
}

int ct_append( char *str, int len ) {
	int text_length;

	text_length = gtk_text_get_length( chat_text );

	gtk_text_set_point( chat_text, text_length );
	gtk_text_insert( chat_text, chat_font, &chat_color,
		NULL, (gchar *)str, len );

	remove_extra_lines();

	return( 0 );
}

int ct_append_fixed( char *str, int len ) {
	GdkFont *orig_font_p;

	orig_font_p = chat_font;
	chat_font = gdk_font_load( "fixed" );
	ct_append( str, len );
	gdk_font_unref( chat_font );
	chat_font = orig_font_p;

	return( 0 );
}

int ct_freeze() {
	gtk_text_freeze( chat_text );

	return( 0 );
}

int ct_thaw() {
	gtk_text_thaw( chat_text );

	return( 0 );
}

int ct_clear() {
	GtkEditable *text_editable;
	int text_length;

	gtk_text_freeze( chat_text );
	text_length = gtk_text_get_length( chat_text );
	text_editable = &(chat_text->editable);
	gtk_editable_delete_text( GTK_EDITABLE(text_editable), 0,
		text_length );
	gtk_text_thaw( chat_text );

	return( 0 );
}

GdkColor *ct_get_color() {
	return( &chat_color );
}

GdkFont *ct_get_font() {
	return( chat_font );
}

int ct_set_color( int r, int g, int b ) {
	chat_color.red   = r * ( 65535 / 255 );
	chat_color.green = g * ( 65535 / 255 );
	chat_color.blue  = b * ( 65535 / 255 );

	gdk_colormap_alloc_color( gtk_widget_get_colormap(GTK_WIDGET(chat_text)),
	        &chat_color, FALSE, TRUE);

	return( 0 );
}

int ct_set_font( GdkFont *new_font ) {
	chat_font = new_font;

	return( 0 );
}

int ct_set_max_lines( int lines ) {
	if ( lines >= 0 )
		max_lines = lines;
	else
		max_lines = 0;

	return( 0 );
}

int ct_capture_to_file( char *filename ) {
	FILE *fp;
	int  length;
	int  i;
	gchar ch;
	time_t time_now;

	fp = fopen( filename, "ab" );

	if ( ! fp )
		return( 0 );

	length = gtk_text_get_length( chat_text );

	time_now = time(NULL);

	fprintf( fp, "Captured at: %s", ctime(&time_now) );
	fprintf( fp, "-------------------------------------\n" );

	for( i = 0; i < length; i++ ) {
		ch = GTK_TEXT_INDEX( chat_text, i );
		fwrite( &ch, 1, 1, fp );
	}

	fprintf( fp, "\n" );
	fclose( fp );

	return( 0 );
}

int ct_can_do_pixmaps( void ) {
	return( 0 );
}

int ct_append_pixmap( char *filename ) {
	return( 0 );
}

gboolean
on_chat_text_clicked                   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
		if ( check_clicked_text( get_word_at_cursor( chat_text ) )) {
			return( TRUE );
		}
	}

	return FALSE;
}

#endif  /* GYACH_INT_TEXT */
